# -*- coding: utf-8 -*-
"""Tests for core functions."""

import base64
from datetime import timedelta
import pickle
import pprint
import zlib
from io import BytesIO
from unittest.mock import patch, Mock
from urllib.parse import urlparse

from click.testing import CliRunner
import pytest
from flask import request, session
from flask_login import current_user, login_user, logout_user
from peewee import fn
from werkzeug.datastructures import ImmutableMultiDict

from orcid_hub import authcontroller, create_hub_administrator, login_provider, utils
from orcid_hub.models import (Affiliation, OrcidAuthorizeCall, OrcidToken, Organisation, OrgInfo,
                              OrgInvitation, Role, User, UserInvitation, UserOrg)


def test_create_hub_administrator(app):
    """Test creation of the Hub administrators."""
    org = app.data["org"]
    runner = CliRunner()
    runner.invoke(create_hub_administrator, ["root000@test.ac.nz"])
    assert User.select().where(User.email == "root000@test.ac.nz").exists()
    assert Organisation.select().where(Organisation.name == "ORCID Hub").exists()
    runner.invoke(create_hub_administrator, ["root000@test.ac.nz", "-O", "NEW ORGANISATION #0"])
    assert Organisation.select().where(Organisation.name == "NEW ORGANISATION #0").exists()
    assert User.get(email="root000@test.ac.nz").organisation.name == "NEW ORGANISATION #0"

    runner.invoke(create_hub_administrator, ["root001@test.ac.nz", "-O", "NEW ORG"])
    assert Organisation.select().where(Organisation.name == "NEW ORG").exists()
    assert User.get(email="root001@test.ac.nz").organisation.name == "NEW ORG"

    org_count = Organisation.select().count()
    runner.invoke(create_hub_administrator, ["root002@test.ac.nz", "-O", org.name])
    assert Organisation.select().count() == org_count
    assert User.get(email="root002@test.ac.nz").organisation.name == org.name

    runner.invoke(create_hub_administrator, ["root010@test.ac.nz", "-I", "INTERNAL NAME 111"])
    assert User.select().where(User.email == "root010@test.ac.nz").exists()
    assert Organisation.select().where(Organisation.name == "ORCID Hub",
                                       Organisation.tuakiri_name == "INTERNAL NAME 111").exists()
    assert User.get(email="root010@test.ac.nz").organisation.tuakiri_name == "INTERNAL NAME 111"

    runner.invoke(create_hub_administrator,
                  ["root011@test.ac.nz", "-O", "NEW ORG", "-I", "INTERNAL NAME 222"])
    assert Organisation.select().where(Organisation.name == "NEW ORG",
                                       Organisation.tuakiri_name == "INTERNAL NAME 222").exists()
    assert User.get(email="root011@test.ac.nz").organisation.tuakiri_name == "INTERNAL NAME 222"

    org_count = Organisation.select().count()
    runner.invoke(create_hub_administrator, ["root012@test.ac.nz", "-O", org.name, "-I", "INTERNAL NAME 333"])
    assert Organisation.select().count() == org_count
    assert User.get(email="root012@test.ac.nz").organisation.tuakiri_name == "INTERNAL NAME 333"


def test_index(client, monkeypatch):
    """Test the landing page."""
    client.application.config["EXTERNAL_SP"] = "https://some.externar.sp/SP"
    resp = client.get("/")
    assert resp.status_code == 200
    assert b"https://some.externar.sp/SP" in resp.data
    assert b"<!DOCTYPE html>" in resp.data
    assert b"Royal Society of New Zealand" in resp.data, \
        "'Royal Society of New Zealand' should be present on the index page."
    client.application.config["EXTERNAL_SP"] = None


def get_response(request_ctx):
    """Return response within the request context.  It should be used with the request context."""
    # TODO: it can be replace with a single line: resp = ctx.app.full_dispatch_request()
    app = request_ctx.app
    # call the before funcs
    resp = app.preprocess_request()
    if resp is not None:
        response = app.make_response(resp)
    else:
        # do the main dispatch
        resp = app.dispatch_request()
        response = app.make_response(resp)

        # now do the after funcs
        response = app.process_response(response)

    return response


def test_login(request_ctx):
    """Test login function."""
    with request_ctx("/") as ctx:
        test_user = User(
            name="TEST USER", email="test@test.test.net", confirmed=True)
        login_user(test_user, remember=True)

        resp = get_response(ctx)
        assert b"<!DOCTYPE html>" in resp.data, "Expected HTML content"
        assert b"TEST USER" in resp.data, "Expected to have the user name on the page"
        assert b"test@test.test.net" in resp.data, "Expected to have the user email on the page"

        logout_user()
        resp = get_response(ctx)
        assert b"test@test.test.net" not in resp.data


def test_org_switch(client):
    """Test organisation switching."""
    user = User.get(orcid=User.select(fn.COUNT(User.orcid).alias("id_count"), User.orcid).group_by(
        User.orcid).having(fn.COUNT(User.orcid) > 1).naive().first().orcid)
    user_orgs = UserOrg.select().join(User).where(User.orcid == user.orcid)
    new_org = Organisation.select().where(Organisation.id.not_in([uo.org_id for uo in user_orgs])).first()
    UserOrg.create(user=user, org=new_org, affiliations=0)

    resp = client.login(user, follow_redirects=True)
    assert user.email.encode() in resp.data
    assert len(user.org_links) > 1
    assert current_user == user

    # Nothing changes if it is the same organisation
    uo = user.userorg_set.where(UserOrg.org_id == user.organisation_id).first()
    resp = client.get(f"/select/user_org/{uo.id}", follow_redirects=True)
    assert User.get(user.id).organisation_id == user.organisation_id
    assert user.email.encode() in resp.data

    # The current org changes if it's a dirrerent org on the list
    uo = user.userorg_set.where(UserOrg.org_id != user.organisation_id).first()
    resp = client.get(f"/select/user_org/{uo.id}", follow_redirects=True)
    assert User.get(user.id).organisation_id != user.organisation_id
    assert User.get(user.id).organisation_id == uo.org_id

    for ol in user.org_links:
        assert ol.org.name.encode() in resp.data
        if UserOrg.get(ol.id).user.id != user.id:
            next_ol = ol

    # Shoud be a totally different user account:
    resp = client.get(f"/select/user_org/{next_ol.id}", follow_redirects=True)
    next_user = UserOrg.get(next_ol.id).user
    assert next_user.id != user.id


@pytest.mark.parametrize("url",
                         ["/link", "/auth", "/pyinfo", "/invite/organisation", "/invite/user"])
def test_access(url, client):
    """Test access to the app for unauthorized user."""
    resp = client.get(url)
    assert resp.status_code == 302
    assert "Location" in resp.headers, pprint.pformat(resp.headers, indent=4)
    assert "next=" in resp.location
    resp = client.get(url, follow_redirects=True)
    assert resp.status_code == 200
    assert b"<!DOCTYPE html>" in resp.data, "Expected HTML content"
    assert b"Please log in to access this page." in resp.data


def test_tuakiri_login(client):
    """
    Test logging attempt via Shibboleth.

    After getting logged in a new user entry shoulg be created.
    """
    data = {
        "Auedupersonsharedtoken": "ABC123",
        "Sn": "LAST NAME/SURNAME/FAMILY NAME",
        'Givenname': "FIRST NAME/GIVEN NAME",
        "Mail": "user@test.test.net",
        "O": "ORGANISATION 123",
        "Displayname": "TEST USER FROM 123",
        "Unscoped-Affiliation": "staff",
        "Eppn": "user@test.test.net"
    }

    resp = client.get("/sso/login", headers=data)

    assert resp.status_code == 302
    u = User.get(email="user@test.test.net")
    assert u.name == "TEST USER FROM 123", "Expected to have the user in the DB"
    assert u.first_name == "FIRST NAME/GIVEN NAME"
    assert u.last_name == "LAST NAME/SURNAME/FAMILY NAME"


def test_sso_loging_with_external_sp(client, mocker):
    """Test with EXTERNAL_SP."""
    data = {
        "Auedupersonsharedtoken": "ABC123",
        "Sn": "LAST NAME/SURNAME/FAMILY NAME",
        'Givenname': "FIRST NAME/GIVEN NAME",
        "Mail": "test_external_sp@test.ac.nz;secondory@test.edu",
        "O": client.data["org"].name,
        "Displayname": "TEST USER #42",
        "Unscoped-Affiliation": "staff",
        "Eppn": "user@test.test.net"
    }

    client.application.config["EXTERNAL_SP"] = "https://exernal.ac.nz/SP"
    resp = client.get("/index")
    assert b"https://exernal.ac.nz/SP" in resp.data
    get = mocker.patch(
        "requests.get",
        return_value=Mock(text=base64.b64encode(zlib.compress(pickle.dumps(data)))))

    resp = client.get("/sso/login", follow_redirects=True)
    get.assert_called_once()
    assert b"TEST USER #42" in resp.data
    assert b"test_external_sp@test.ac.nz" in resp.data
    u = User.get(email="test_external_sp@test.ac.nz")
    assert u.name == "TEST USER #42"


def test_tuakiri_login_usgin_eppn(client):
    """Test logging attempt via Shibboleth using differt values to identify the user."""
    org = Organisation(tuakiri_name="ORGANISATION 123ABC")
    org.save()
    user = User.create(
        email="something_else@test.test.net", eppn="eppn123@test.test.net", roles=Role.RESEARCHER)
    user.save()

    resp = client.get(
        "/Tuakiri/login",
        headers={
            "Auedupersonsharedtoken": "ABC123",
            "Sn": "LAST NAME/SURNAME/FAMILY NAME",
            'Givenname': "FIRST NAME/GIVEN NAME",
            "Mail": "user123@test.test.net",
            "O": "ORGANISATION 123ABC",
            "Displayname": "TEST USER FROM 123",
            "Unscoped-Affiliation": "staff",
            "Eppn": "eppn123@test.test.net"
        })

    assert resp.status_code == 302
    u = User.get(eppn="eppn123@test.test.net")
    assert u.email == "something_else@test.test.net"
    assert u.name == "TEST USER FROM 123", "Expected to have the user in the DB"
    assert u.first_name == "FIRST NAME/GIVEN NAME"
    assert u.last_name == "LAST NAME/SURNAME/FAMILY NAME"


def test_tuakiri_login_wo_org(client):
    """
    Test logging attempt via Shibboleth.

    If a user logs in from an organisation that isn't
    onboared, the user should be informed about that and
    redirected to the login page.
    """
    resp = client.get(
        "/Tuakiri/login",
        headers={
            "Auedupersonsharedtoken": "ABC999",
            "Sn": "LAST NAME/SURNAME/FAMILY NAME",
            'Givenname': "FIRST NAME/GIVEN NAME",
            "Mail": "user@test.test.net",
            "O": "INCOGNITO",
            "Displayname": "TEST USER FROM UNKNOWN",
            "Unscoped-Affiliation": "staff",
            "Eppn": "user@test.test.net"
        },
        follow_redirects=True)

    u = User.get(email="user@test.test.net")
    assert u is not None
    assert u.eppn == "user@test.test.net"
    assert b"Your organisation (INCOGNITO) is not yet using the Hub, " \
           b"see your Technical Contact for a timeline" in resp.data


def test_tuakiri_login_with_org(client):
    """
    Test logging attempt via Shibboleth.

    If a user logs in from an organisation that isn't
    onboared, the user should be informed about that and
    redirected to the login page.
    """
    org = client.data.get("org")
    resp = client.get(
        "/Tuakiri/login",
        headers={
            "Auedupersonsharedtoken": "ABC111",
            "Sn": "LAST NAME/SURNAME/FAMILY NAME",
            'Givenname': "FIRST NAME/GIVEN NAME",
            "Mail": "user111@test.test.net",
            "O": "THE ORGANISATION",
            "Displayname": "TEST USER FROM THE ORGANISATION",
            "Unscoped-Affiliation": "staff",
            "Eppn": "user@test.test.net"
        },
        follow_redirects=True)

    u = User.get(email="user111@test.test.net")
    assert u.organisation == org
    assert org in u.organisations
    assert b"Your organisation (THE ORGANISATION) is not onboarded" not in resp.data
    uo = UserOrg.get(user=u, org=org)
    assert not uo.is_admin


def test_tuakiri_login_by_techical_contact_organisation_not_onboarded(client):
    """Test logging attempt by technical contact when organisation is not onboarded."""
    org = Organisation(name="Org112", tuakiri_name="Org112", confirmed=False, is_email_sent=True)
    u = User(
        email="user1113@test.test.net", confirmed=True, roles=Role.TECHNICAL, organisation=org)
    org.tech_contact = u
    org.save()

    UserOrg(user=u, org=org, is_admin=True)
    resp = client.get(
        "/Tuakiri/login",
        headers={
            "Auedupersonsharedtoken": "ABC11s1",
            "Sn": "LAST NAME/SURNAME/FAMILY NAME",
            'Givenname': "FIRST NAME/GIVEN NAME",
            "Mail": "user1113@test.test.net",
            "O": "Org112",
            "Displayname": "TEST USER FROM THE Org112",
            "Unscoped-Affiliation": "student",
            "Eppn": "user11137@test.test.net"
        },
        follow_redirects=True)

    assert u.organisation == org
    assert not org.confirmed
    assert u.is_tech_contact_of(org)
    assert resp.status_code == 200
    assert b"<!DOCTYPE html>" in resp.data, "Expected HTML content"


def test_confirmation_token(app):
    """Test generate_confirmation_token and confirm_token."""
    app.config['SECRET_KEY'] = "SECRET"
    token = utils.generate_confirmation_token("TEST@ORGANISATION.COM")
    assert utils.confirm_token(token) == "TEST@ORGANISATION.COM"

    app.config['SECRET_KEY'] = "COMPROMISED SECRET"
    with pytest.raises(Exception) as ex_info:
        utils.confirm_token(token)
    # Got exception
    assert "does not match" in ex_info.value.message


def test_login_provider_load_user(request_ctx):  # noqa: D103
    """Test to load user."""
    u = User(
        email="test123@test.test.net",
        name="TEST USER",
        roles=Role.RESEARCHER,
        orcid=None,
        confirmed=True)
    u.save()

    user = login_provider.load_user(u.id)
    assert user == u
    assert login_provider.load_user(9999999) is None

    with request_ctx("/"):

        login_user(u)
        resp = login_provider.roles_required(Role.RESEARCHER)(lambda: "SUCCESS")()
        assert resp == "SUCCESS"

        resp = login_provider.roles_required(Role.SUPERUSER)(lambda: "SUCCESS")()
        assert resp != "SUCCESS"
        assert resp.status_code == 302
        assert resp.location.startswith("/")


def test_onboard_org(client):
    """Test to organisation onboarding."""
    org = Organisation.create(
        name="THE ORGANISATION:test_onboard_org",
        tuakiri_name="THE ORGANISATION:test_onboard_org",
        confirmed=False,
        orcid_client_id="CLIENT ID",
        orcid_secret="Client Secret",
        city="CITY",
        country="COUNTRY",
        disambiguated_id="ID",
        disambiguation_source="SOURCE",
        is_email_sent=True)
    u = User.create(
        email="test123@test.test.net",
        name="TEST USER",
        roles=Role.TECHNICAL,
        orcid="123",
        confirmed=True,
        organisation=org)
    second_user = User.create(
        email="test1234@test.test.net",
        name="TEST USER",
        roles=Role.ADMIN,
        orcid="1243",
        confirmed=True,
        organisation=org)
    UserOrg.create(user=second_user, org=org, is_admin=True)
    org_info = OrgInfo.create(
        name="THE ORGANISATION:test_onboard_org", tuakiri_name="THE ORGANISATION:test_onboard_org")
    org.tech_contact = u
    org_info.save()
    org.save()

    client.login_root()
    with patch("orcid_hub.utils.send_email"):
        resp = client.post(
            "/invite/organisation",
            data=dict(org_name="A NEW ORGANISATION", org_email="test_abc_123@test.test.net"))
        assert User.select().where(User.email == "test_abc_123@test.test.net").exists()
        resp = client.post(
            "/invite/organisation",
            data=dict(
                org_name="A NEW ORGANISATION", org_email="test12345@test.test.net",
                tech_contact='y'))
        assert User.select().where(User.email == "test12345@test.test.net").exists()
    org = Organisation.get(name="A NEW ORGANISATION")
    user = User.get(email="test12345@test.test.net")
    assert user.name is None
    assert org.tech_contact is None
    client.logout()

    resp = client.login(
        user,
        **{
            "Sn": "TECHNICAL",
            "Givenname": "CONTACT",
            "Displayname": "Test User",
            "shib_O": "NEW ORGANISATION"
        },
        follow_redirects=True)
    user = User.get(email="test12345@test.test.net")
    org = user.organisation
    assert user.is_tech_contact_of(org)
    resp = client.get("/confirm/organisation")
    assert resp.status_code == 200
    org = Organisation.get(org.id)
    assert b"<!DOCTYPE html>" in resp.data, "Expected HTML content"
    assert b"Take me to ORCID to obtain my Client ID and Client Secret" in resp.data

    with patch("orcid_hub.authcontroller.requests") as requests:
        requests.post.return_value = Mock(data=b'XXXX', status_code=200)
        resp = client.post(
            "/confirm/organisation",
            data={
                "orcid_client_id": "APP-1234567890ABCDEF",
                "orcid_secret": "12345678-1234-1234-1234-1234567890ab",
                "country": "NZ",
                "city": "Auckland",
                "disambiguated_id": "XYZ123",
                "disambiguation_source": "XYZ",
                "name": org.name,
                "email": user.email,
            })
    assert resp.status_code == 302
    url = urlparse(resp.location).path
    assert url == "/link"
    resp = client.get(url)
    client.logout()
    org = Organisation.get(org.id)
    assert org.disambiguated_id == "XYZ123"
    assert org.disambiguation_source == "XYZ"
    assert org.orcid_client_id == "APP-1234567890ABCDEF"
    assert org.orcid_secret == "12345678-1234-1234-1234-1234567890ab"

    user = User.get(email="test_abc_123@test.test.net")
    resp = client.login(
        user,
        **{
            "Sn": "NEW ORGANISATION",
            "Givenname": "ADMINISTRATOR",
            "Displayname": "Admin User",
            "shib_O": "NEW ORGANISATION"
        },
        follow_redirects=True)
    assert b"Take me to ORCID to allow A NEW ORGANISATION permission to access my ORCID record" in resp.data

    resp = client.get("/confirm/organisation")
    assert resp.status_code == 302
    assert urlparse(resp.location).path == "/admin/viewmembers/"

    resp = client.get("/admin/viewmembers/")
    assert b"test12345@test.test.net" in resp.data

    resp = client.get("/admin/viewmembers/export/csv/")
    assert resp.headers["Content-Type"] == "text/csv; charset=utf-8"
    assert b"test12345@test.test.net" in resp.data
    assert b"test_abc_123@test.test.net" in resp.data


@patch("orcid_hub.utils.send_email")
def test_invite_tech_contact(send_email, client):
    """Test on-boarding of an org."""
    client.login_root()
    email = "tech.contact@a.new.org"
    client.post(
        "/invite/organisation",
        data={
            "org_name": "A NEW ORGANISATION",
            "org_email": email,
            "tech_contact": "y",
        })
    u = User.get(email=email)
    oi = OrgInvitation.get(invitee=u)

    assert not u.confirmed
    assert oi.org.name == "A NEW ORGANISATION"
    assert oi.org.tech_contact is None
    send_email.assert_called_once()
    client.logout()

    # Test invited user login:
    client.login(u, **{"Sn": "Surname", "Givenname": "Givenname", "Displayname": "Test User"})
    u = User.get(email=email)
    assert u.confirmed
    assert u.organisation.tech_contact == u


def test_logout(client):
    """Test to logout."""
    org = Organisation.create(
        name="THE ORGANISATION:test_logout",
        tuakiri_name="University of Auckland",
        confirmed=True,
        is_email_sent=True)
    user = User.create(
        email="test@test.test.net",
        name="TEST USER",
        roles=Role.TECHNICAL,
        confirmed=True,
        organisation=org)

    client.login(user)
    resp = client.get("/logout")
    # UoA user:
    assert resp.status_code == 302
    assert "Shibboleth.sso" in resp.location
    assert "uoa-slo" in resp.location

    org.tuakiri_name = org.name
    org.save()
    client.login(user)
    resp = client.get("/logout")
    # non-UoA user:
    assert resp.status_code == 302
    assert "Shibboleth.sso" in resp.location
    assert "uoa-slo" not in resp.location


def test_orcid_login(client):
    """Test login from orcid."""
    org = Organisation.get(name="THE ORGANISATION")
    u = User.create(
        email="test123_test_orcid_login@test.test.net",
        name="TEST USER",
        roles=Role.TECHNICAL,
        orcid="123",
        confirmed=True,
        organisation=org)
    user_org = UserOrg.create(user=u, org=org, is_admin=True)
    token = "TOKEN-1234567"
    ui = UserInvitation.create(org=org, invitee=u, email=u.email, token=token)
    resp = client.get(f"/orcid/login/{token}")
    assert resp.status_code == 200
    orcid_authorize = OrcidAuthorizeCall.get(method="GET")
    assert "&email=test123_test_orcid_login%40test.test.net" in orcid_authorize.url
    ui.created_at -= timedelta(days=100)
    ui.save()
    resp = client.get(f"/orcid/login/{token}")
    assert resp.status_code == 302
    url = urlparse(resp.location)
    assert url.path == '/'
    # Testing the expired token flow for researcher
    user_org.is_admin = False
    user_org.save()
    resp = client.get(f"/orcid/login/{token}")
    assert resp.status_code == 302
    url = urlparse(resp.location)
    assert url.path == '/'


def fetch_token_mock(self,
                     token_url=None,
                     code=None,
                     authorization_response=None,
                     body='',
                     auth=None,
                     username=None,
                     password=None,
                     method='POST',
                     timeout=None,
                     headers=None,
                     verify=True,
                     proxies=None,
                     **kwargs):
    """Mock token fetching api call."""
    token = {
        'orcid': '123',
        'name': 'ros',
        'access_token': 'xyz',
        'refresh_token': 'xyz',
        'scope': '/activities/update',
        'expires_in': '12121'
    }
    return token


def get_record_mock(self, orcid=None, **kwargs):
    """Mock record api call."""
    request.data = '{"noemail": "sstest123@test.test.net"}'
    return request


@patch("orcid_hub.OAuth2Session.fetch_token", side_effect=fetch_token_mock)
@patch("orcid_hub.orcid_client.MemberAPIV20Api.view_emails", side_effect=get_record_mock)
def test_orcid_login_callback_admin_flow(patch, patch2, request_ctx):
    """Test login from orcid callback function for Organisation Technical contact."""
    org = Organisation.create(
        name="THE ORGANISATION:test_orcid_login_callback_admin_flow",
        tuakiri_name="THE ORGANISATION:test_orcid_login_callback_admin_flow",
        confirmed=False,
        orcid_client_id="CLIENT ID",
        orcid_secret="Client Secret",
        city="CITY",
        country="COUNTRY",
        disambiguated_id="ID",
        disambiguation_source="SOURCE",
        is_email_sent=True)
    u = User.create(
        email="test123@test.test.net",
        roles=Role.TECHNICAL,
        orcid="123",
        confirmed=False,
        organisation=org)
    UserOrg.create(user=u, org=org, is_admin=True)
    token = utils.generate_confirmation_token(email=u.email, org=org.name)

    with request_ctx() as resp:
        request.args = {"invitation_token": token, "state": "xyz"}
        session['oauth_state'] = "xyz"
        resp = authcontroller.orcid_login_callback(request)
        assert resp.status_code == 302
        assert resp.location.startswith("/")
    with request_ctx() as respx:
        request.args = {"invitation_token": token, "state": "xyzabc"}
        session['oauth_state'] = "xyz"
        respx = authcontroller.orcid_login_callback(request)
        assert respx.status_code == 302
        assert respx.location.startswith("/")
    with request_ctx() as resp:
        request.args = {"invitation_token": token, "state": "xyz", "error": "access_denied"}
        session['oauth_state'] = "xyz"
        resp = authcontroller.orcid_login_callback(request)
        assert resp.status_code == 302
        assert resp.location.startswith("/")
    with request_ctx() as ct:
        token = utils.generate_confirmation_token(email=u.email, org=None)
        request.args = {"invitation_token": token, "state": "xyz"}
        session['oauth_state'] = "xyz"
        ctxx = authcontroller.orcid_login_callback(request)
        assert ctxx.status_code == 302
        assert ctxx.location.startswith("/")
    with request_ctx() as ctxxx:
        request.args = {"invitation_token": token, "state": "xyzabc"}
        session['oauth_state'] = "xyz"
        ctxxx = authcontroller.orcid_login_callback(request)
        assert ctxxx.status_code == 302
        assert ctxxx.location.startswith("/")
    with request_ctx() as cttxx:
        request.args = {"invitation_token": token, "state": "xyz", "error": "access_denied"}
        session['oauth_state'] = "xyz"
        cttxx = authcontroller.orcid_login_callback(request)
        assert cttxx.status_code == 302
        assert cttxx.location.startswith("/")
    with request_ctx() as ct:
        token = utils.generate_confirmation_token(email=u.email, org=None)
        request.args = {"invitation_token": token, "state": "xyz"}
        session['oauth_state'] = "xyz"
        ct = authcontroller.orcid_login_callback(request)
        assert ct.status_code == 302
        assert ct.location.startswith("/")
    with request_ctx():
        request.args = {"invitation_token": None, "state": "xyz"}
        session['oauth_state'] = "xyz"
        ct = authcontroller.orcid_login_callback(request)
        assert ct.status_code == 302
        assert ct.location.startswith("/")
    with request_ctx():
        # Test case for catching general exception: invitation token here is integer, so an exception will be thrown.
        request.args = {"invitation_token": 123, "state": "xyz"}
        session['oauth_state'] = "xyz"
        ct = authcontroller.orcid_login_callback(request)
        assert ct.status_code == 302
        assert ct.location.startswith("/")
    with request_ctx():
        # User login via orcid, where organisation is not confirmed.
        u.orcid = "123"
        u.save()
        request.args = {"invitation_token": None, "state": "xyz-about"}
        session['oauth_state'] = "xyz-about"
        resp = authcontroller.orcid_login_callback(request)
        assert resp.status_code == 302
        assert resp.location.startswith("/about")
    with request_ctx():
        # User login via orcid, where organisation is confirmed, so showing viewmembers page.
        org.tech_contact = u
        org.confirmed = True
        org.save()
        request.args = {"invitation_token": None, "state": "xyz"}
        session['oauth_state'] = "xyz"
        resp = authcontroller.orcid_login_callback(request)
        assert resp.status_code == 302
        assert resp.location.startswith("/admin/viewmembers/")
    with request_ctx():
        # User login via orcid, where organisation is not confirmed and user is tech, so showing confirm org page.
        org.confirmed = False
        org.save()
        request.args = {"invitation_token": None, "state": "xyz"}
        session['oauth_state'] = "xyz"
        resp = authcontroller.orcid_login_callback(request)
        assert resp.status_code == 302
        assert resp.location.startswith("/confirm/organisation")


def affiliation_mock(
        self=None,
        affiliation=None,
        role=None,
        department=None,
        org_name=None,
        # NB! affiliation_record has 'organisation' field for organisation name
        organisation=None,
        city=None,
        state=None,
        region=None,
        country=None,
        disambiguated_id=None,
        disambiguation_source=None,
        start_date=None,
        end_date=None,
        put_code=None,
        initial=False,
        *args,
        **kwargs):
    """Mock record api call."""
    return "xyz"


@patch("orcid_hub.OAuth2Session.fetch_token", side_effect=fetch_token_mock)
@patch(
    "orcid_hub.orcid_client.MemberAPI.create_or_update_affiliation", side_effect=affiliation_mock)
def test_orcid_login_callback_researcher_flow(patch, patch2, request_ctx):
    """Test login from orcid callback function for researcher and display profile."""
    org = Organisation.create(
        name="THE ORGANISATION:test_orcid_login_callback_researcher_flow",
        tuakiri_name="THE ORGANISATION:test_orcid_login_callback_researcher_flow",
        confirmed=True,
        orcid_client_id="CLIENT ID",
        orcid_secret="Client Secret",
        city="CITY",
        country="COUNTRY",
        disambiguated_id="ID",
        disambiguation_source="SOURCE",
        is_email_sent=True)
    u = User.create(
        email="test123@test.test.net",
        name="TEST USER",
        roles=Role.RESEARCHER,
        orcid="123",
        confirmed=True,
        organisation=org)
    UserOrg.create(user=u, org=org, is_admin=False)
    token = utils.generate_confirmation_token(email=u.email, org=org.name)
    UserInvitation.create(email=u.email, token=token, affiliations=Affiliation.EMP)
    OrcidToken.create(user=u, org=org, scope="/read-limited,/activities/update")
    with request_ctx():
        request.args = {"invitation_token": token, "state": "xyz"}
        session['oauth_state'] = "xyz"
        resp = authcontroller.orcid_login_callback(request)
        assert resp.status_code == 302
        # display profile
        assert resp.location.startswith("/profile")


def test_select_user_org(request_ctx):
    """Test organisation switch of current user."""
    org = Organisation.create(
        name="THE ORGANISATION:test_select_user_org",
        tuakiri_name="THE ORGANISATION:test_select_user_org",
        confirmed=True,
        orcid_client_id="CLIENT ID",
        orcid_secret="Client Secret",
        city="CITY",
        country="COUNTRY",
        disambiguated_id="ID",
        disambiguation_source="SOURCE",
        is_email_sent=True)
    org2 = Organisation.create(
        name="THE ORGANISATION2:test_select_user_org",
        tuakiri_name="THE ORGANISATION2:test_select_user_org",
        confirmed=True,
        orcid_client_id="CLIENT ID",
        orcid_secret="Client Secret",
        city="CITY",
        country="COUNTRY",
        disambiguated_id="ID",
        disambiguation_source="SOURCE",
        is_email_sent=True)

    user = User.create(
        email="test123@test.test.net",
        name="TEST USER",
        roles=Role.TECHNICAL,
        orcid="123",
        confirmed=True,
        organisation=org)
    org.save()
    org2.save()
    user.save()
    UserOrg.create(user=user, org=org, is_admin=True)
    user_org2 = UserOrg.create(user=user, org=org2, is_admin=True)
    with request_ctx(f"/select/user_org/{user_org2.id}") as ctx:
        login_user(user, remember=True)
        resp = ctx.app.full_dispatch_request()
        assert resp.status_code == 302
        assert user.organisation_id != org.id
        # Current users organisation has been changes from 1 to 2
        assert user.organisation_id == org2.id


def test_shib_sp(client):
    """Test shibboleth SP."""
    resp = client.get("/Tuakiri/SP?key=123ABC&url=/profile", headers={"USER": "TEST123ABC"})
    assert resp.status_code == 302
    assert "/profile" in resp.location

    resp = client.get("/sp/attributes/123ABC")
    assert resp.status_code == 200
    data = pickle.loads(zlib.decompress(base64.b64decode(resp.data)))
    assert data["User"] == "TEST123ABC"

    resp = client.get("/Tuakiri/SP?key=123&url=https://harmfull.one/profile")
    assert resp.status_code == 403


def test_get_attributes(request_ctx):
    """Test shibboleth SP."""
    key = "xyz"
    with request_ctx("/sp/attributes/" + key) as ctxx:
        resp = ctxx.app.full_dispatch_request()
        # No such file name 'xyz'
        assert resp.status_code == 403


def test_link(request_ctx):
    """Test orcid profile linking."""
    org = Organisation.create(
        name="THE ORGANISATION:test_link",
        tuakiri_name="THE ORGANISATION:test_link",
        confirmed=True,
        orcid_client_id="CLIENT ID",
        orcid_secret="Client Secret",
        city="CITY",
        country="COUNTRY",
        disambiguated_id="ID",
        disambiguation_source="SOURCE",
        is_email_sent=True)
    user = User.create(
        email="test123@test.test.net",
        name="TEST USER",
        roles=Role.TECHNICAL,
        orcid="123",
        confirmed=True,
        organisation=org)
    UserOrg.create(user=user, org=org, is_admin=True)
    with request_ctx("/link") as ctx:
        login_user(user, remember=True)
        request.args = ImmutableMultiDict([('error', 'access_denied')])
        resp = ctx.app.full_dispatch_request()
        assert resp.status_code == 200
        assert b"<!DOCTYPE html>" in resp.data, "Expected HTML content"


@pytest.mark.parametrize("url", ["/faq", "/about"])
def test_faq_and_about(client, url):
    """Test faq and about page path traversal security issue."""
    resp = client.get(url + "?malicious_code")
    assert resp.status_code == 403
    resp = client.get(url)
    assert resp.status_code == 200


def test_orcid_callback(client):
    """Test orcid researcher deny flow."""
    org = Organisation.create(
        name="THE ORGANISATION:test_orcid_callback",
        tuakiri_name="THE ORGANISATION:test_orcid_callback",
        confirmed=True,
        orcid_client_id="CLIENT ID",
        orcid_secret="Client Secret",
        city="CITY",
        country="COUNTRY",
        disambiguated_id="ID",
        disambiguation_source="SOURCE",
        is_email_sent=True)
    user = User.create(
        email="test123_test_orcid_callback@test.test.net",
        name="TEST USER",
        roles=Role.TECHNICAL,
        orcid="123",
        confirmed=True,
        organisation=org)
    UserOrg.create(user=user, org=org, is_admin=True)

    client.login(user)
    resp = client.get("/auth?error=access_denied&login=2")
    assert resp.status_code == 302
    assert "/link" in resp.location


def test_login0(client):
    """Test login from orcid."""
    from orcid_hub import current_user
    resp = client.get("/login0")
    assert resp.status_code == 401

    u = User.select().where(User.orcid.is_null(False)).first()
    email = u.email
    import itsdangerous
    signature = itsdangerous.Signer(client.application.secret_key).get_signature(email).decode()
    auth = email + ':' + signature

    resp = client.get(f"/login0/{auth}")
    assert resp.status_code == 302
    assert current_user.email == email

    resp = client.get("/login0", headers={"Authorization": auth})
    assert resp.status_code == 302
    assert current_user.email == email

    from base64 import b64encode
    resp = client.get("/login0", headers={"Authorization": b64encode(auth.encode()).decode()})
    assert resp.status_code == 302
    assert current_user.email == email


def test_load_test_data(app):
    """Test load test data generation."""
    client = app.test_client()
    client.login_root()

    resp = client.get("/test-data")
    assert resp.status_code == 200
    assert b"Load Test Data Generation" in resp.data

    resp = client.post("/test-data?user_count=123")
    assert resp.data.count(b'\n') == 123

    resp = client.post("/test-data")
    assert resp.data.count(b'\n') == 400

    import itsdangerous
    signature = itsdangerous.Signer(
        client.application.secret_key).get_signature("abc123@gmail.com")
    resp = client.post(
        "/test-data",
        data={
            "file_": (
                BytesIO(
                    b"""nks98991100099999981,paw01,ros1,abc123@gmail.com,The University of Auckland,Rosha1
nks011,paw01,ros1,2orcid100110009001@gmail.com,The University of Auckland,Rosha1,abc456@auckland.ac.nz,faculty
"""),
                "DATA.csv",
            ),
        })
    assert resp.status_code == 200
    assert signature in resp.data
    assert "DATA_SIGNED.csv" in resp.headers["Content-Disposition"]

    resp = client.post(
        "/test-data",
        data={
            "file_": (
                BytesIO(
                    "abc123@gmail.com\tUniversity\nanother@gmail.com\tThe University of Auckland".
                    encode("utf-16")),
                "DATA_WITH_TABS.csv",
            ),
        })
    assert resp.status_code == 200
    assert signature in resp.data
    assert resp.data.count(b'\n') == 2
    assert "DATA_WITH_TABS_SIGNED.csv" in resp.headers["Content-Disposition"]

    resp = client.post(
        "/test-data",
        data={
            "file_": (
                BytesIO(b"email\tname\nabc123@gmail.com\tUniversity\nanother@gmail.com\tThe University of Auckland"),
                "DATA_WITH_TABS_AND_HEADERS.csv",
            ),
        })
    assert resp.status_code == 200
    assert signature in resp.data
    assert resp.data.count(b'\n') == 2

    assert "DATA_WITH_TABS_AND_HEADERS_SIGNED.csv" in resp.headers["Content-Disposition"]

    client.logout()



#!/usr/bin/env python
# -*-coding: utf-8 -*-
"""Collects and renders Python and Flask application evironment and configuration."""

import cgi
import os
import platform
import socket
import sys

from . import app

optional_modules_list = [
    'Cookie', 'mod_wsgi', 'psycopg2', 'zlib', 'gzip', 'bz2', 'zipfile', 'tarfile', 'ldap',
    'socket', 'audioop', 'curses', 'imageop', 'aifc', 'sunau', 'wave', 'chunk', 'colorsys',
    'rgbimg', 'imghdr', 'sndhdr', 'ossaudiodev', 'sunaudiodev', 'adodbapi', 'cx_Oracle', 'ibm_db',
    'mxODBC', 'MySQLdb', 'pgdb', 'PyDO', 'sapdbapi', 'sqlite3'
]


def _info():
    for i in optional_modules_list:
        try:
            module = __import__(i)
            sys.modules[i] = module
            globals()[i] = module
        except Exception:
            pass

    info = {}
    info['python_version'] = platform.python_version()
    info['system_info'] = get_system_info()
    info['py_internals'] = get_py_internals()
    info['os_internals'] = get_os_internals()
    info['envvars'] = get_envvars()
    info['database_info'] = get_database_info()
    info['compression_info'] = get_compression_info()
    info['socket_info'] = get_socket_info()
    info['multimedia_info'] = get_multimedia_info()
    info["app_config"] = app.config
    return info


def get_system_info():  # noqa: D103
    system_info = []

    distname = platform.linux_distribution()[0]
    version = platform.linux_distribution()[1]
    if distname != '' and version != '':
        os_version = '%s %s (%s %s)' % (platform.system(), platform.release(), distname, version)
    else:
        os_version = '%s %s' % (platform.system(), platform.release())
    system_info.append(('OS Version', os_version))

    if hasattr(os, 'path'):
        system_info.append(('OS Path', os.environ['PATH']))

    if hasattr(sys, 'version'):
        system_info.append(('Python Version', sys.version))

    if hasattr(sys, 'subversion'):
        system_info.append(('Python Subversion', sys.subversion[0]))

    if hasattr(sys, 'prefix'):
        system_info.append(('Python Prefix', sys.prefix))

    if hasattr(sys, 'path'):
        system_info.append(('Python Path', sys.path))

    if hasattr(sys, 'executable'):
        system_info.append(('Python Executable', sys.executable))

    if hasattr(sys, 'api_version'):
        system_info.append(('Python API', sys.api_version))

    system_info.append(('Build Date', platform.python_build()[1]))
    system_info.append(('Compiler', platform.python_compiler()))

    return system_info


def get_py_internals():  # noqa: D103
    py_internals = []
    if hasattr(sys, 'builtin_module_names'):
        py_internals.append(('Built-in Modules', ', '.join(sys.builtin_module_names)))
    py_internals.append(('Byte Order', sys.byteorder + ' endian'))

    if hasattr(sys, 'getcheckinterval'):
        py_internals.append(('Check Interval', sys.getcheckinterval()))

    if hasattr(sys, 'getfilesystemencoding'):
        py_internals.append(('File System Encoding', sys.getfilesystemencoding()))

    max_integer_size = str(sys.maxsize) + ' (%s)' % \
                                          hex(sys.maxsize).upper()
    py_internals.append(('Maximum Integer Size', max_integer_size))

    if hasattr(sys, 'getrecursionlimit'):
        py_internals.append(('Maximum Recursion Depth', sys.getrecursionlimit()))

    if hasattr(sys, 'tracebacklimit'):
        traceback_limit = sys.tracebacklimit
    else:
        traceback_limit = 1000
    py_internals.append(('Maximum Traceback Limit', traceback_limit))

    py_internals.append(('Maximum Code Point', sys.maxunicode))
    return py_internals


def get_os_internals():  # noqa: D103
    os_internals = []
    if hasattr(os, 'getcwd'):
        os_internals.append(("Current Working Directory", os.getcwd()))
    if hasattr(os, 'getegid'):
        os_internals.append(("Effective Group ID", os.getegid()))

    if hasattr(os, 'geteuid'):
        os_internals.append(("Effective User ID", os.geteuid()))

    if hasattr(os, 'getgid'):
        os_internals.append(("Group ID", os.getgid()))

    if hasattr(os, 'getuid'):
        os_internals.append(("User ID", os.getuid()))

    if hasattr(os, 'getgroups'):
        os_internals.append(("Group Membership", ', '.join(map(str, os.getgroups()))))

    if hasattr(os, 'linesep'):
        os_internals.append(("Line Seperator", repr(os.linesep)[1:-1]))

    if hasattr(os, 'pathsep'):
        os_internals.append(("Path Seperator", os.pathsep))

    if hasattr(os, 'getloadavg'):
        os_internals.append(("Load Avarage", ', '.join(
            map(lambda x: str(round(x, 2)), os.getloadavg()))))

    return os_internals


def get_envvars():  # noqa: D103
    return [(k, cgi.escape(v, quote=True)) for k, v in os.environ.items()]


def get_database_info():  # noqa: D103
    database_info = []
    database_info.append(('DB2/Informix (ibm_db)', is_imported('ibm_db')))
    database_info.append(('MSSQL (adodbapi)', is_imported('adodbapi')))
    database_info.append(('MySQL (MySQL-Python)', is_imported('MySQLdb')))
    database_info.append(('ODBC (mxODBC)', is_imported('mxODBC')))
    database_info.append(('Oracle (cx_Oracle)', is_imported('cx_Oracle')))
    database_info.append(('PostgreSQL (PyGreSQL)', is_imported('pgdb')))
    database_info.append(('PostgreSQL (Psycopg)', is_imported('psycopg2')))
    database_info.append(('Python Data Objects (PyDO)', is_imported('PyDO')))
    database_info.append(('SAP DB (sapdbapi)', is_imported('sapdbapi')))
    database_info.append(('SQLite3', is_imported('sqlite3')))
    return database_info


def get_compression_info():  # noqa: D103
    compression_info = []
    compression_info.append(('Bzip2 Support', is_imported('bz2')))
    compression_info.append(('Gzip Support', is_imported('gzip')))
    compression_info.append(('Tar Support', is_imported('tarfile')))
    compression_info.append(('Zip Support', is_imported('zipfile')))
    compression_info.append(('Zlib Support', is_imported('zlib')))
    return compression_info


def get_socket_info():  # noqa: D103
    socket_info = []
    socket_info.append(('Hostname', socket.gethostname()))
    socket_info.append(('Hostname (fully qualified)',
                        socket.gethostbyaddr(socket.gethostname())[0]))
    try:
        socket_info.append(('IP Address', socket.gethostbyname(socket.gethostname())))
    except Exception:
        pass
    socket_info.append(('IPv6 Support', getattr(socket, 'has_ipv6', False)))
    socket_info.append(('SSL Support', hasattr(socket, 'ssl')))
    return socket_info


def get_multimedia_info():  # noqa: D103
    multimedia_info = []
    multimedia_info.append(('AIFF Support', is_imported('aifc')))
    multimedia_info.append(('Color System Conversion Support', is_imported('colorsys')))
    multimedia_info.append(('curses Support', is_imported('curses')))
    multimedia_info.append(('IFF Chunk Support', is_imported('chunk')))
    multimedia_info.append(('Image Header Support', is_imported('imghdr')))
    multimedia_info.append(('OSS Audio Device Support', is_imported('ossaudiodev')))
    multimedia_info.append(('Raw Audio Support', is_imported('audioop')))
    multimedia_info.append(('Raw Image Support', is_imported('imageop')))
    multimedia_info.append(('SGI RGB Support', is_imported('rgbimg')))
    multimedia_info.append(('Sound Header Support', is_imported('sndhdr')))
    multimedia_info.append(('Sun Audio Device Support', is_imported('sunaudiodev')))
    multimedia_info.append(('Sun AU Support', is_imported('sunau')))
    multimedia_info.append(('Wave Support', is_imported('wave')))
    return multimedia_info


def is_imported(module):  # noqa: D103
    if module in sys.modules:
        return 'enabled'
    return 'disabled'


info = _info()



# -*- coding: utf-8 -*-
"""Application configuration."""

from os import environ, getenv, path

ENV = getenv("ENV", "dev")
SHIBBOLETH_DISABLED = getenv("SHIBBOLETH_DISABLED")

ORCID_API_HOST_URL = "https://api.sandbox.orcid.org/" if ENV != "prod" else "https://api.orcid.org/"
ORCID_API_VERSION = "v2.0"
ORCID_API_BASE = ORCID_API_HOST_URL + ORCID_API_VERSION + '/'
ORCID_BASE_URL = "https://sandbox.orcid.org/" if ENV != "prod" else "https://orcid.org/"

# NB! Set up the key. See: http://flask.pocoo.org/docs/latest/quickstart/#sessions
SECRET_KEY = getenv("SECRET_KEY", b'\xe3\x94a\x14-sT`\x92\x8a0\x16\r\xe1zb')
SENTRY_DSN = getenv("SENTRY_DSN")

# TODO: Soon to be depricated:
SALT = getenv("TOKEN_PASSWORD_SALT")

# NZ ORCIDHUB API client ID and secret
ORCID_CLIENT_ID = getenv("ORCID_CLIENT_ID")
ORCID_CLIENT_SECRET = getenv("ORCID_CLIENT_SECRET")

# Change the URL as per the enviornment
AUTHORIZATION_BASE_URL = 'https://sandbox.orcid.org/oauth/authorize' \
    if ENV != "prod" else "https://orcid.org/oauth/authorize"
TOKEN_URL = 'https://sandbox.orcid.org/oauth/token' if ENV != "prod" else "https://orcid.org/oauth/token"

# Database connection url
DATABASE_URL = getenv("DATABASE_URL", "sqlite:///data.db")
BACKUP_DATABASE_URL = getenv("BACKUP_DATABASE_URL")
LOAD_TEST = getenv("LOAD_TEST")

# NB! Disable in production
if ENV in ("dev0", ):
    DEBUG = '1'
    MAIL_DEBUG = '1'
    TESTING = True
    DEBUG_TB_INTERCEPT_REDIRECTS = False
    DEBUG_TB_PROFILER_ENABLED = True
    OAUTHLIB_INSECURE_TRANSPORT = '1'
    environ['OAUTHLIB_INSECURE_TRANSPORT'] = '1'
    TEMPLATES_AUTO_RELOAD = True
    # EXPLAIN_TEMPLATE_LOADING = True

OAUTH2_PROVIDER_TOKEN_EXPIRES_IN = 86400  # Default Bearer token expires time, default is 3600.

# add mail server config
MAIL_PORT = int(getenv("MAIL_PORT", 25))
MAIL_SUPPRESS_SEND = False
MAIL_DEFAULT_SENDER = getenv("MAIL_DEFAULT_SENDER", "no-reply@orcidhub.org.nz")
MAIL_SERVER = getenv("MAIL_SERVER", "gateway")

MEMBER_API_FORM_BASE_URL = "https://orcid.org/content/register-client-application-sandbox" \
    if ENV != "prod" else "https://orcid.org/content/register-client-application-production-trusted-party"

NOTE_ORCID = 'An NZ ORCID Hub integration for'
CRED_TYPE_PREMIUM = 2
APP_NAME = 'NZ ORCID HUB'
APP_DESCRIPTION = 'This is an ORCID integration through the NZ ORCID HUB connecting'
APP_URL = "https://" + (ENV + ".orcidhub.org.nz" if ENV != "prod" else "orcidhub.org.nz")
SEED_HUB_ADMIN = getenv("SEED_HUB_ADMIN", "rad42@mailinator.com")

# External Shibboleth SP login URL (e.g., https://test.orcidhub.org.nz/Tuakiri/login)
EXTERNAL_SP = getenv("EXTERNAL_SP") if ENV != "prod" else None

DEFAULT_COUNTRY = "NZ"

if ENV == "dev":
    GA_TRACKING_ID = "UA-99022483-1"
elif ENV == "test":
    GA_TRACKING_ID = "UA-99022483-2"
elif ENV == "dev0":
    GA_TRACKING_ID = "UA-99022483-3"
else:
    GA_TRACKING_ID = "UA-99022483-4"

DEFAULT_EMAIL_TEMPLATE = """<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8">
    <title>{SUBJECT}</title>
    <meta name="keywords" content="ORCID HUB,New Zealand,NZ,orcid"/>
  </head>
  <body>
    <div style="font-family: Arial, Verdana, Georgia, and Times New Roman; background-color: white; color: black;">
      <table style="vertical-align: middle; background-color: black; color: white; width: 100%;">
        <tr>
          <td>&nbsp;</td>
          <td align="right">
            <img src="{LOGO}" title="NZ ORCID Hub" alt="NZ ORCID Hub" style="display:block;" />
          </td>
        </tr>
      </table>
      {MESSAGE}
      <p>If you received this email in error, or you have questions about the responsibilities
      involved, please contact: <a href="mailto:orcid@royalsociety.org.nz">
      orcid@royalsociety.org.nz</a></p>
      <p>For information about ORCID go to the
      <a href="https://royalsociety.org.nz/orcid-in-new-zealand/what-is-orcid/orcid-for-researchers/">What is ORCID?</a>
       page of our website.</p>
      <hr>
      <p>This email was sent to <a href="mailto:{EMAIL}">{EMAIL}</a></p>
      <!--  Footer Details -->
      <table style="vertical-align: top; background-color: black; color: white; width: 100%; margin-top: 25px;">
        <tr>
          <td>
            <p style="vertical-align: top; padding-left: 15px;">
            Contact details for the NZ ORCID Hub<br>
            Phone: (04) 472 7421<br>
            PO Box 598, Wellington 6140<br>
            <b><a style="text-decoration: none; color: white;"
                  href="mailto:orcid@royalsociety.org.nz">orcid@royalsociety.org.nz</a></b>
            </p>
          </td>
          <td style="vertical-align: top;">
            <p class="copyright"><a href="https://creativecommons.org/licenses/by/4.0/"
            target="_blank"><img
            src="{BASE_URL}/static/images/CC-BY-icon-80x15.png" alt="CC-BY" /></a></p>
          </td>
        </tr>
      </table>
    </div>
  </body>
</html>
"""

DKIP_KEY_PATH = path.join(path.dirname(path.relpath(path.relpath(__file__))), ".keys", "dkim.key")

# RQ:
RQ_REDIS_URL = getenv("RQ_REDIS_URL")
RQ_QUEUE_CLASS = "orcid_hub.queuing.ThrottledQueue"

# rq-dashboard config:
RQ_POLL_INTERVAL = 5000  #: Web interface poll period for updates in ms
WEB_BACKGROUND = "gray"



# coding: utf-8

"""
    ORCID Member

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: Latest
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

# import models into sdk package
from .models.activities_summary import ActivitiesSummary
from .models.address import Address
from .models.amount import Amount
from .models.authorization_url import AuthorizationUrl
from .models.bulk_element import BulkElement
from .models.citation import Citation
from .models.contributor import Contributor
from .models.contributor_attributes import ContributorAttributes
from .models.contributor_email import ContributorEmail
from .models.contributor_orcid import ContributorOrcid
from .models.country import Country
from .models.created_date import CreatedDate
from .models.credit_name import CreditName
from .models.day import Day
from .models.disambiguated_organization import DisambiguatedOrganization
from .models.education import Education
from .models.education_summary import EducationSummary
from .models.educations import Educations
from .models.employment import Employment
from .models.employment_summary import EmploymentSummary
from .models.employments import Employments
from .models.external_id import ExternalID
from .models.external_i_ds import ExternalIDs
from .models.funding import Funding
from .models.funding_contributor import FundingContributor
from .models.funding_contributor_attributes import FundingContributorAttributes
from .models.funding_contributors import FundingContributors
from .models.funding_group import FundingGroup
from .models.funding_summary import FundingSummary
from .models.funding_title import FundingTitle
from .models.fundings import Fundings
from .models.fuzzy_date import FuzzyDate
from .models.group_id_record import GroupIdRecord
from .models.group_id_records import GroupIdRecords
from .models.item import Item
from .models.items import Items
from .models.keyword import Keyword
from .models.last_modified_date import LastModifiedDate
from .models.month import Month
from .models.notification import Notification
from .models.notification_permission import NotificationPermission
from .models.organization import Organization
from .models.organization_address import OrganizationAddress
from .models.organization_defined_funding_sub_type import OrganizationDefinedFundingSubType
from .models.other_name import OtherName
from .models.peer_review import PeerReview
from .models.peer_review_group import PeerReviewGroup
from .models.peer_review_summary import PeerReviewSummary
from .models.peer_reviews import PeerReviews
from .models.person_external_identifier import PersonExternalIdentifier
from .models.publication_date import PublicationDate
from .models.researcher_url import ResearcherUrl
from .models.source import Source
from .models.source_client_id import SourceClientId
from .models.source_name import SourceName
from .models.source_orcid import SourceOrcid
from .models.subtitle import Subtitle
from .models.title import Title
from .models.translated_title import TranslatedTitle
from .models.url import Url
from .models.work import Work
from .models.work_bulk import WorkBulk
from .models.work_contributors import WorkContributors
from .models.work_group import WorkGroup
from .models.work_summary import WorkSummary
from .models.work_title import WorkTitle
from .models.works import Works
from .models.year import Year

# import apis into sdk package
from .apis.member_apiv20_api import MemberAPIV20Api
from .apis.member_apiv21_api import MemberAPIV21Api

# import ApiClient
from .api_client import ApiClient

from .configuration import Configuration

configuration = Configuration()
